package org.jboss.fuse.qa;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.MatcherAssert.assertThat;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import org.apache.qpid.jms.JmsConnectionFactory;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author jknetl
 */
public class SelectorTest {

    static final Logger logger = LoggerFactory.getLogger(SelectorTest.class);

    public static final String QUEUE_NAME = "test.queue";
    private static final String PORT = System.getProperty("amqp.port", "5555");
    public static final String URL = "amqp://localhost:" + PORT;
    public static final String USER_NAME = "admin";
    public static final String PASSWORD = "admin";


    @Test
    public void testSelectors() throws JMSException {
        ConnectionFactory cf = new JmsConnectionFactory(USER_NAME, PASSWORD, URL);

        Connection connection = null;
        try {
            connection = cf.createConnection();
            Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
            Destination destination = session.createQueue(QUEUE_NAME);
            connection.start();
            MessageProducer producer = session.createProducer(destination);


            String myHeader = "seqNumber";

            for (int i = 0; i < 10; i++) {
                TextMessage message = session.createTextMessage("hello world");
                message.setIntProperty(myHeader, i);
                producer.send(message);
            }

            int seqSelected = 3;
            String selector = String.format("%s = %d", myHeader, seqSelected);
            MessageConsumer consumer = session.createConsumer(destination, selector);
            boolean nullMessage = false;
            int received = 0;
            while (!nullMessage) {
                Message m = consumer.receive(2000);
                if (m != null) {
                    received++;
                    assertThat(m.getIntProperty(myHeader), is(seqSelected));
                } else {
                    nullMessage = true;
                }
            }

            logger.info("Message received: {}", received);
            assertThat(received, is(1));
        } catch (JMSException e) {
            e.printStackTrace();
        } finally {
            if (connection != null) {
                connection.close();
            }
        }
    }
}
